import sys
import subprocess

def install_packages(packages):
    for package in ['pip'] + packages:
        print('============================================================')
        print(f"✔️ Installing '{package}'...")
        proc = subprocess.Popen([sys.executable, '-m', 'pip', 'install', '--upgrade', package], stdout=subprocess.PIPE)
        output = proc.stdout.read().decode('utf-8')
        print(output)
        print('============================================================')



import pathlib as paths
import zipfile

from maat_machine.pprint import displayitwell
import maat_machine.filesystem as mt_fs

def unzip_to_directory(zip_file: paths.Path, destination_directory: paths.Path):
    assert zip_file.is_file()
    assert mt_fs.directory_exists(destination_directory)
    with zipfile.ZipFile(str(zip_file), 'r') as zip_reference:
        zip_reference.extractall(destination_directory)
        extracted_files = zip_reference.namelist()
        displayitwell(f"ZIP file \"{zip_file}\" is extracted to \"{destination_directory}\"; Extracted files count: {len(extracted_files)}", font_style='oblique', font_weight='bold')



from types import NoneType
from typing import Union

import pathlib as paths
import zipfile

import json
import pandas as pd
import numpy as np
import tensorflow as tf
from tensorflow import keras as ktf
import keras
from keras import models as mktf
from keras import layers as lktf
from keras import optimizers as oktf
from keras import preprocessing as ppktf
from keras import utils as uktf
from keras import losses as lsktf
import tensorflow.keras.preprocessing.image as tkpi
import sklearn.base as sk_base
import sklearn.model_selection as sk_modsel

from keras import regularizers

from PIL import Image as pillow_images

from IPython.display import display


class CNNCustomClassifier(sk_base.BaseEstimator, sk_base.ClassifierMixin):
    def __init__(self,
                 num_classes, input_shape,
                 epochs, batch_size,
                 optimizer, loss,
                 validation_split,

                 file_path_column_name,
                 label_column_name,

                 conv_layer_array,
                 conv_kernel_size_default,
                 conv_activation, conv_activation_parameter,
                 conv_pool_size, conv_dropout,

                 dense_layer_array,
                 dense_activation, dense_activation_parameter,
                 dense_dropout):
        self.num_classes = num_classes
        self.input_shape = input_shape
        self.epochs = epochs
        self.batch_size = batch_size
        self.optimizer = optimizer
        self.loss = loss
        self.validation_split=validation_split

        self.file_path_column_name = file_path_column_name
        self.label_column_name = label_column_name

        self.conv_layer_array = conv_layer_array
        self.conv_kernel_size_default = conv_kernel_size_default
        self.conv_activation = conv_activation
        self.conv_activation_parameter = conv_activation_parameter
        self.conv_pool_size = conv_pool_size
        self.conv_dropout = conv_dropout

        self.dense_layer_array = dense_layer_array
        self.dense_activation = dense_activation
        self.dense_activation_parameter = dense_activation_parameter
        self.dense_dropout = dense_dropout

        self.model = None

    def create_cnn_model(self):
        model = keras.Sequential()
        model.add(lktf.Input(shape=self.input_shape, name='input'))

        for i, conv_layer_config in list(enumerate(self.conv_layer_array)):
            conv_filters_i, conv_kernel_i, layers_i = conv_layer_config
            assert 'c' in layers_i
#             if i == 0:
#                 model.add(lktf.Conv2D(filters=conv_filters_i, kernel_size=(1, 1), padding='same'))

            print(f"Conv Layer config: {conv_layer_config}")

            model.add(lktf.Conv2D(
                filters=conv_filters_i, kernel_size=conv_kernel_i,
                padding='same',
#                 kernel_regularizer=regularizers.L2(1e-3),
#                 activity_regularizer=regularizers.L2(1e-3),
                name=f"conv2d-{i}_0"
            ))
#             if 'b' in layers_i:
#                 model.add(lktf.BatchNormalization(name=f"batch-{i}_0"))
            model.add(lktf.ReLU(name=f"relu-{i}_0"))
            conv2d_count = layers_i.count('c')
            if conv2d_count > 1:
                for c_i in range(0, conv2d_count-1):
                    model.add(lktf.Conv2D(
                        filters=conv_filters_i, kernel_size=conv_kernel_i,
                        padding='same',
        #                 kernel_regularizer=regularizers.L2(1e-3),
        #                 activity_regularizer=regularizers.L2(1e-3),
                        name=f"conv2d-{i}_{c_i+1}"
                    ))
#                     if 'b' in layers_i:
#                         model.add(lktf.BatchNormalization(name=f"batch-{i}_{c_i+1}"))
                    model.add(lktf.ReLU(name=f"relu-{i}_{c_i+1}"))
        #             model.add(self._obtain_activation(self.conv_activation, self.conv_activation_parameter))
            if 'b' in layers_i:
                model.add(lktf.BatchNormalization(name=f"batch-{i}"))
            if 'p' in layers_i:
                model.add(lktf.MaxPooling2D(pool_size=self.conv_pool_size, padding='same', name=f"max_pooling_2d-{i}"))
            if 'd' in layers_i:
                model.add(lktf.Dropout(self.conv_dropout, name=f"dropout-{i}"))

        model.add(lktf.Flatten(name='flatten'))

        for i, dense_layer_config in list(enumerate(self.dense_layer_array)):
            dense_layer_size_i, layers_i = dense_layer_config
            assert 'e' in layers_i

            if 'b' in layers_i:
                model.add(lktf.BatchNormalization(name=f"dense-batch-{i}"))

            model.add(lktf.Dense(
                dense_layer_size_i,
#                 kernel_regularizer=regularizers.L2(1e-3),
#                 activity_regularizer=regularizers.L2(1e-3),
                name=f"dense-{i}"
            ))
#             model.add(self._obtain_activation(self.dense_activation, self.dense_activation_parameter))
            model.add(lktf.ReLU(name=f"dense-relu-{i}"))

            if 'd' in layers_i:
                model.add(lktf.Dropout(self.dense_dropout, name=f"dense-dropout-{i}"))

        model.add(lktf.Dense(self.num_classes, activation='softmax', name='output'))

        model.compile(
            optimizer=self.optimizer,
            loss=self.loss,
            metrics=['accuracy'],
        )
        self.model = model

        return model

    def _obtain_activation(self, name: str, negative_slope: Union[int, NoneType] = None):
        if name == 'relu':
            return lktf.ReLU(negative_slope=0.0 if negative_slope is None else negative_slope)
        elif name == 'lrelu':
            return lktf.LeakyReLU(negative_slope=0.3 if negative_slope is None else negative_slope)

    def fit(self, features: pd.DataFrame, labels: pd.Series):
        assert (features.index == labels.index).all()
        assert self.file_path_column_name in features
        assert self.label_column_name == labels.name
        assert self.model is not None

        # ImageDataGenerator сам делает преобразование меток, если class_mode='categorical'
        # encoded_labels = pd.Series(
        #     self.one_hot_encoder.transform(labels.values.reshape(-1, 1)).toarray().tolist(),
        #     index=labels.index,
        #     name=self.label_column_name
        # )
        data_frame = pd.concat([features, labels], axis=1)
        display(data_frame.sample(10))
        display(np.unique(labels))

        image_data_generator = tkpi.ImageDataGenerator(
            rescale=1.0/255,
            validation_split=self.validation_split
        )
        train_generator = image_data_generator.flow_from_dataframe(
            dataframe=data_frame,
            x_col=self.file_path_column_name,
            y_col=self.label_column_name,
            color_mode="rgb",
            class_mode="categorical",
            target_size=(self.input_shape[0], self.input_shape[1]),
            batch_size=self.batch_size,
            seed=137,
            suffle=True,
            subset='training'
        )
        validation_generator = image_data_generator.flow_from_dataframe(
            dataframe=data_frame,
            x_col=self.file_path_column_name,
            y_col=self.label_column_name,
            color_mode="rgb",
            class_mode="categorical",
            target_size=(self.input_shape[0], self.input_shape[1]),
            batch_size=self.batch_size,
            seed=137,
            suffle=True,
            subset='validation'
        )

        result = self.model.fit(
            train_generator,
            epochs=self.epochs,
            validation_data=validation_generator,
        )
        return result

    def predict_proba_from_dataframe(self, features: pd.DataFrame):
        assert self.file_path_column_name in features
        image_data_generator = tkpi.ImageDataGenerator(rescale=1.0/255)
        test_generator = image_data_generator.flow_from_dataframe(
            dataframe=features,
            directory=None,
            x_col=self.file_path_column_name,
            y_col=None,
            batch_size=self.batch_size,
            seed=42,
            shuffle=False,
            color_mode="rgb",
            class_mode=None,
            target_size=(self.input_shape[0], self.input_shape[1]),
        )
        return self.model.predict(test_generator)
    
    def predict_from_dataframe(self, features: pd.DataFrame):
        labels_predicted_proba = self.predict_proba_from_dataframe(features)
        predicted_labels = np.argmax(labels_predicted_proba, axis=1)
        return predicted_labels
    
    def score_from_dataframe(self, features: pd.DataFrame, labels_true: pd.Series):
        labels_predicted = self.predict_from_dataframe(features)
        accuracy = np.mean(labels_predicted == labels_true) * 100
        return accuracy
    
    def predict_proba_from_pil_image(self, image: pillow_images.Image):
        target_width = self.input_shape[1]
        target_height = self.input_shape[0]
        test_image = image.resize((target_width, target_height))
        test_image = test_image.convert('RGB')
        image_array = np.array(test_image, dtype='float64') / 255.0
        image_array = np.expand_dims(image_array, axis=0)
        return self.model.predict(image_array) 

    def predict_from_pil_image(self, image: pillow_images.Image):
        labels_predicted_proba = self.predict_proba_from_pil_image(image)
        predicted_label = np.argmax(labels_predicted_proba, axis=1)
        return predicted_label

    def save(self, directory_path: paths.Path):
        assert directory_path is not None
        assert directory_path.is_dir()

        path_weights = directory_path / 'model.weights.h5'
        self.model.save_weights(path_weights, overwrite=True, save_format='h5')

        path_model = directory_path / 'model.tf'
        self.model.save(path_model, overwrite=True, save_format='tf')

        self_dictionary = vars(self).copy()
        self_dictionary.pop('model', None)
        self_dictionary.pop('optimizer', None)
        serialized_wrapper_json = json.dumps(self_dictionary)

        path_wrapper = directory_path / 'model.wrapper.json'
        with open(path_wrapper, 'w') as f:
            f.write(serialized_wrapper_json)

        path_model_archive = directory_path / 'model.zip'
        with zipfile.ZipFile(path_model_archive, 'w') as zip_reference:
            zip_reference.write(directory_path / 'model.tf', 'model.tf')
            zip_reference.write(directory_path / 'model.weights.h5', 'model.weights.h5')
            zip_reference.write(directory_path / 'model.wrapper.json', 'model.wrapper.json')

    @staticmethod
    def load(model_archive_path: paths.Path, extaction_directory_path: paths.Path):
        assert model_archive_path is not None
        assert model_archive_path.is_file()
        assert model_archive_path.suffix == '.zip'
        assert extaction_directory_path is not None
        assert extaction_directory_path.is_dir()

        archive_files = {
            'weights': 'model.weights.h5',
            'model': 'model.tf',
            'wrapper': 'model.wrapper.json',
        }

        path_model_archive = extaction_directory_path / 'model.zip'
        with zipfile.ZipFile(path_model_archive, 'r') as zip_reference:
            for item_name, archive_file_name in archive_files.items():
                assert archive_file_name in zip_reference.namelist()

        with zipfile.ZipFile(path_model_archive, 'r') as zip_reference:
            zip_reference.extractall(extaction_directory_path)

        model = mktf.load_model(extaction_directory_path / archive_files['model'])
        model.load_weights(extaction_directory_path / archive_files['weights'])

        with open(extaction_directory_path / archive_files['wrapper'], 'r') as f:
            wrapper_dict = json.load(f)

        wrapper = CNNCustomClassifier(**wrapper_dict)
        wrapper.model = model
        return wrapper




import pathlib as paths
import cv2 as cv
import pandas as pd
from tqdm.auto import tqdm


class ImageSizeGenerator:
    def __init__(self, image_files: list):
        assert len(image_files) > 0
        assert all(isinstance(f, paths.Path) for f in image_files)
        self.image_files = image_files

    def __iter__(self):
        for file_path in self.image_files:
            img = cv.imread(str(file_path))
            height, width, channels = img.shape
            yield (width, height, channels)


def calculate_image_statistics(image_files: list, recommend_by: str = 'width'):
    image_size_generator = ImageSizeGenerator(image_files)

    widths = []
    heights = []
    channel_counts = []
    for width, height, channels in image_size_generator:
        widths.append(width)
        heights.append(height)
        channel_counts.append(channels)

    min_width, max_width = min(widths), max(widths)
    min_height, max_height = min(heights), max(heights)
    avg_width, avg_height = sum(widths) / len(widths), sum(heights) / len(heights)

    recommended_width = -1
    recommended_height = -1
#     resize_with = 'height' if avg_height > avg_width else 'width'
    resize_with = recommend_by
    if resize_with == 'height':
        recommended_height = int(round(avg_height, 0))
        scale_ratios = [recommended_height / height for height in heights]
        recommended_width = int(round(max(width * scale_ratios[i] for i, width in enumerate(widths)), 0))
    elif resize_with == 'width':
        recommended_width = int(round(avg_width, 0))
        scale_ratios = [recommended_width / width for width in widths]
        recommended_height = int(round(max(height * scale_ratios[i] for i, height in enumerate(heights)), 0))
    else:
        raise NotImplementedError()

    return (min_width, max_width, min_height, max_height, avg_width, avg_height, channel_counts, recommended_width, recommended_height)

class ImageResizingGenerator:
    def __init__(self, image_series, target_fixed_height, target_fixed_width, target_channels, output_directory, pbar):
        self.image_series = image_series
        self.target_fixed_height = target_fixed_height
        self.target_fixed_width = target_fixed_width
        self.output_directory = output_directory
        self.target_channels = target_channels
        self.pbar = pbar

    def generate_images(self):
        for idx, image_path in self.image_series.items():
            img = cv.imread(str(image_path))
            # img = cv.cvtColor(img, cv.COLOR_BGR2RGB)
            img_to_change = None
            if self.target_channels == 1:
                img_to_change = cv.cvtColor(img, cv.COLOR_BGR2GRAY)  # Convert to grayscale
            else:
                img_to_change = img
#             current_height, current_width, _ = img.shape

#             ratio = self.target_fixed_height / current_height
            resized_img = cv.resize(
                img_to_change, (self.target_fixed_width, self.target_fixed_height),
                interpolation=cv.INTER_LANCZOS4
            )

            new_img = resized_img

            new_image_path = self.output_directory / f'image_{idx}.jpg'
            self.pbar.set_description(f"Resized 🖼-#{idx} {resized_img.shape}➡{new_img.shape}🖬➡\"{new_image_path}\"")

            cv.imwrite(str(new_image_path), new_img)

            self.pbar.update(1)
            yield idx, new_image_path


def dataset_load_image(file_path: paths.Path, target_channels: int):
    image = cv.imread(str(file_path))
    if target_channels == 1:
        image = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
    else:
        image = cv.cvtColor(image, cv.COLOR_BGR2RGB)
    return image



from types import NoneType
from typing import Union

from IPython.display import display
from IPython.display import HTML


def displayit(value: str):
    """
    Displays the given HTML value using IPython's display function.

    Parameters:
        value (str): The HTML value to be displayed.

    Returns:
        None
    """
    display(HTML(value))


def displayitwell(value: str, color: str = 'black',
                  font_style: str = 'normal', font_weight: str = 'normal', text_decoration: Union[str, NoneType] = None,
                  font_size: str = '1em', font_family='default', inline: bool = True):
    """
    A function that displays the 'value' with specified styling options.

    Parameters:
        value (str): The content to be displayed.
        color (str): The color of the text (default is 'black').
        font_style (str): The style of the font (default is 'normal').
        font_weight (str): The weight of the font (default is 'normal').
        text_decoration (Union[str, NoneType]): The text decoration style (default is None).
        font_size (str): The size of the font (default is '1em').
        font_family (str): The font family (default is 'default').
        inline (bool): Whether to display the content inline or in a block (default is True).
    """
    tag = 'span' if inline else 'div'
    text_decoration_value = f" text-decoration: {text_decoration};" if text_decoration is not None else ''
    displayit(f"<{tag} style='color: {color}; font-style: {font_style}; font-weight: {font_weight};{text_decoration_value} font-size: {font_size}; font-family: {font_family if font_family != 'default' else ''}'>{value}</{tag}>")


def dict_to_html(dictionary):
    """
    Converts a dictionary into an HTML-formatted string.

    Parameters:
        dictionary (dict): The dictionary to be converted to HTML.

    Returns:
        str: The HTML-formatted string representing the dictionary.
    """
    html_output = '<div>{<br>\n'
    for key, value in dictionary.items():
        html_output += f"&nbsp;&nbsp;{key}: {value}<br>\n"
    html_output += '}</div>'
    return html_output


import os
import urllib
import pathlib as paths
from maat_machine.pprint import displayitwell


def print_path_info(path: paths.Path, path_title: str = 'Path'):
    displayitwell(f"{path_title}: \"{path}\"; Absolute: \"{path.absolute()}\"", font_style='oblique', font_weight='bold')


def directory_exists(directory_path: paths.Path):
    return directory_path.exists() and directory_path.is_dir()


def make_directory(directory_path: paths.Path, print_info: bool = True, path_comment: str = ''):
    assert isinstance(directory_path, paths.Path)
    assert directory_path is not None
    if not directory_exists(directory_path):
        directory_path.mkdir(parents=True, exist_ok=True)
    if print_info:
        print_path_info(directory_path, path_comment)


def make_directories(directory_paths: list, print_info: bool = True, path_comment: str = ''):
    assert all(isinstance(path, paths.Path) for path in directory_paths)
    for directory_path in directory_paths:
        make_directory(directory_path, print_info=print_info, path_comment=path_comment)


def make_directories_in_parent(directory_paths: list, parent_directory: paths.Path, print_info: bool = True, path_comment: str = ''):
    assert all(isinstance(path, paths.PurePath) for path in directory_paths)
    assert isinstance(parent_directory, paths.Path)
    new_directory_full_paths = []
    for directory_path in directory_paths:
        new_directory_full_path = parent_directory.joinpath(directory_path)
        make_directory(new_directory_full_path, print_info=print_info, path_comment=path_comment)
        new_directory_full_paths.append(new_directory_full_path)
    return new_directory_full_paths


def list_directory(directory: paths.Path, list_paths: bool = False, sort: bool = True):
    assert directory.is_dir()
    file_list = [(item if list_paths else item.name) for item in directory.iterdir()]
    return sorted(file_list) if sort else file_list


def extract_file_info_from_url(url):
    parsed_url = urllib.parse.urlparse(url)
    file_name = os.path.basename(parsed_url.path)
    file_name_without_extension, file_extension = os.path.splitext(file_name)
    return file_name, file_name_without_extension, file_extension



# Original code: https://github.com/rahulbordoloi/Directory-Tree/blob/main/src/directory_tree.py
# Importing Libraries
from types import NoneType
from typing import Union
from pathlib import Path
import os
import platform
import stat


# Class for Directory Tree Path
class DirectoryPath:
    """
    Python Utility Package that Displays out the Tree Structure of a Particular Directory.
    @author : rahulbordoloi
    """

    # Class Variables [Directions]
    display_Node_Prefix_Middle: str = '├──'
    display_Node_Prefix_Last: str = '└──'
    display_Parent_Prefix_Middle: str = '    '
    display_Parent_Prefix_Last: str = '│   '

    # Constructor
    def __init__(self, path: Path, parent_path=None, is_last: bool=False):

        # Instance Variables [Status of Parent-Node Files]
        self.path: Path = Path(path)
        self.parent = parent_path
        self.is_last: bool = is_last
        if self.parent:
            self.depth: int = self.parent.depth + 1
        else:
            self.depth: int = 0

    # Displaying Names of the Nodes [Parents/Inner Directories]
    @property
    def displayName(self) -> str:
        if self.path.is_dir():
            return self.path.name + '/'
        return self.path.name

    # Building the Tree [Directories-Nodes]
    @classmethod
    def build_tree(cls, root, parent=None, is_last: bool=False, max_files: Union[int, NoneType] = None, max_depth: float=float("inf"), show_hidden: bool=False, ignore_list: list=None) -> str:

        # Resolving `Ignore List`
        if not ignore_list:
            ignore_list: list = []

        # Generator Method to Generate Tree
        root: Path = Path(root)
        rootDirectoryDisplay = cls(path=root, parent_path=parent, is_last=is_last)
        yield rootDirectoryDisplay

        ## Taking out the List of Children [Nodes] Files/Directories
        children: list = sorted(list(entityPath for entityPath in root.iterdir()), key=lambda s: str(s).lower())

        ## Checking for Hidden Entities Flag
        if not show_hidden:
            children: list = [entityPath for entityPath in children if not cls._hidden_files_filtering_(entityPath)]

        # Filter out Entities (Files and Directories) Specified in the `ignore_list`
        children: list = [
            entityPath for entityPath in children
            if not any(entity == entityPath.name or entity == str(entityPath.relative_to(root)) for entity in ignore_list)
        ]

        if max_files is not None:
            child_directories = [path for path in children if path.is_dir()]
            child_files = [path for path in children if path.is_file()][:max_files]
            children = child_directories + child_files

        countNodes: int = 1
        for path in children:
            is_last = countNodes == len(children)
            if path.is_dir() and rootDirectoryDisplay.depth + 1 < max_depth:
                yield from cls.build_tree(
                    root=path,
                    parent=rootDirectoryDisplay,
                    is_last=is_last,
                    max_files=max_files,
                    max_depth=max_depth,
                    show_hidden=show_hidden,
                    ignore_list=ignore_list
                )
            else:
                yield cls(path=path, parent_path=rootDirectoryDisplay, is_last=is_last)
            countNodes += 1

    # Check Condition for Hidden Entities [Files / Directories]
    @classmethod
    def _hidden_files_filtering_(cls, path) -> bool:
        try:
            return bool(os.stat(path).st_file_attributes & stat.FILE_ATTRIBUTE_HIDDEN) or path.stem.startswith(".")
        except:
            return path.stem.startswith(".")

    # Displaying the Tree Path [Directories-Nodes]
    def displayPath(self) -> str:

        # Check for Parent Directory Name
        if self.parent is None:
            return self.displayName

        # Checking for File-Name Prefix in Tree
        filenamePrefix = (DirectoryPath.display_Node_Prefix_Last if self.is_last else DirectoryPath.display_Node_Prefix_Middle)

        # Adding Prefixes to Beautify Output [List]
        parts: list = [f'{filenamePrefix} {self.displayName}']

        # Adding Prefixes up for Parent-Node Directories
        parent = self.parent
        while parent and parent.parent is not None:
            parts.append(
                DirectoryPath.display_Parent_Prefix_Middle if parent.is_last else DirectoryPath.display_Parent_Prefix_Last)
            parent: str = parent.parent

        return ''.join(reversed(parts))


# Display Function to Print Directory Tree
def display_tree(dir_path: str='', string_rep: bool=False, header: bool=False, max_files: Union[int, NoneType] = None, max_depth: float=float("inf"), show_hidden: bool=False,
                 ignore_list: list=None):
    """
    :param dir_path: Root Path of Operation. By Default, Refers to the Current Working Directory
    :param string_rep: Boolean Flag for Direct Console Output or a String Return of the Same. By Default, It Gives out Console Output
    :param header: Boolean Flag for Displaying [OS & Directory Path] Info in the Console. Not Applicable if `string_rep=True`
    :param max_depth: Max Depth of the Directory Tree. By Default, It goes upto the Deepest Directory/File
    :param show_hidden: Boolean Flag for Returning/Displaying Hidden Files/Directories if Value Set to `True`
    :param ignore_list: List of File and Directory Names or Patterns to Ignore
    :return: None if `string_rep=False` else (str)ing Representation of the Tree
    """

    try:
        # Check for Default Argument
        if dir_path:
            dir_path: Path = Path(dir_path)
        else:
            dir_path: Path = Path(os.getcwd())

        # Build Directory Tree
        paths: str = DirectoryPath.build_tree(
            root=dir_path,
            max_files=max_files,
            max_depth=max_depth,
            show_hidden=show_hidden,
            ignore_list=ignore_list
        )

        # Check for String Representation
        if string_rep:

            # String Representation
            stringOutput: str = str()
            for path in paths:
                stringOutput += path.displayPath() + "\n"
            return stringOutput

        else:
            # Just Console Print
            if header:
                print(f'''
$ Operating System : {platform.system()}
$ Path : {Path(dir_path)}

{"*" * 15} Directory Tree {"*" * 15}
''')

            for path in paths:
                print(path.displayPath())

    except Exception as expMessage:
        print(f"Exception Occurred! Failed to Generate Tree:: {expMessage}")



from matplotlib import pyplot as plt


def plot_training_history(history):
    plt.plot(history["accuracy"], label="Доля верных ответов на обучающем наборе")
    plt.plot(history["val_accuracy"], label="Доля верных ответов на проверочном наборе")
    plt.xlabel("Эпоха обучения")
    plt.ylabel("Доля верных ответов")
    plt.legend()
    plt.show()

    plt.plot(history["loss"], label="Ошибка на обучающем наборе")
    plt.plot(history["val_loss"], label="Ошибка на проверочном наборе")
    plt.xlabel("Эпоха обучения")
    plt.ylabel("Ошибка")
    plt.legend()
    plt.show()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['maat_machine', 'maat_machine.model']

package_data = \
{'': ['*']}

install_requires = \
['ipython>=8.24.0,<9.0.0',
 'matplotlib>=3.8.4,<4.0.0',
 'numpy==1.26.4',
 'opencv-python>=4.9.0.80,<5.0.0.0',
 'pandas>=2.2.2,<3.0.0',
 'pillow>=10.3.0,<11.0.0',
 'scikit-learn>=1.4.2,<2.0.0',
 'tensorflow==2.16.1',
 'tqdm>=4.66.4,<5.0.0']

setup_kwargs = {
    'name': 'maat-machine',
    'version': '0.1.1b2',
    'description': 'Yet another package designed to simplify coding for data science and machine learning',
    'long_description': '',
    'author': 'Dmitry Vlasov',
    'author_email': 'dmitry.v.vlasov@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.11.9',
}


setup(**setup_kwargs)



import sys

from tensorflow import config as tf_config

import maat_machine.pprint as mpprn

def print_system_info():
    print(f"Python version: {sys.version}")
    print(f"Version info: {sys.version_info}")
    print(f"Platform: {sys.platform}")


def display_system_info():
    mpprn.displayitwell(f"<hr>🐍 Python version:<br>&emsp;&emsp;&emsp;&emsp;{sys.version}", color='darkorange', font_weight='bold', inline=False)
    mpprn.displayitwell(f"🍃 Version info:<br>&emsp;&emsp;&emsp;&emsp;{sys.version_info}", color='darkblue', font_weight='bold', inline=False)
    mpprn.displayitwell(f"💻 Platform:<br>&emsp;&emsp;&emsp;&emsp;{sys.platform}<hr>", color='darkred', font_weight='bold', inline=False)


def print_gpu_info():
    print("⚙️ GPU Information:")
    devices = tf_config.list_physical_devices('GPU')
    if devices:
        print('\n'.join([f"\t✅ {tf_config.experimental.get_device_details(device).get('device_name', 'Unknown GPU')}" for device in devices]))
    else:
        print("\t🛑 No GPUs found.")


def display_gpu_info():
    mpprn.displayitwell("<hr>⚙️&nbsp;GPU Information:", color='darkgreen', font_weight='bold', inline=False)
    devices = tf_config.list_physical_devices('GPU')
    if devices:
        mpprn.displayitwell(
            '<ul style="margin-top: -0.8em">' \
            + '</li>'.join([f"<li style=\"list-style: none; padding-left: 1em;\">✅&nbsp;{tf_config.experimental.get_device_details(device).get('device_name', 'Unknown GPU')}" for device in devices]) \
            + '</ul><hr>'
        )
    else:
        mpprn.displayitwell("&emsp;&emsp;&emsp;&emsp;🛑&nbsp;No GPUs found.<hr>")


def print_environment_info():
    print_system_info()
    print_gpu_info()


def display_environment_info():
    display_system_info()
    display_gpu_info()